.\"
.\" sec version 2.0 - sec.pl.man
.\" simple event correlation tool
.\"
.\" Copyright (C) 2000-2001 Risto Vaarandi
.\"
.\" This program is free software; you can redistribute it and/or
.\" modify it under the terms of the GNU General Public License
.\" as published by the Free Software Foundation; either version 2
.\" of the License, or (at your option) any later version.
.\"
.\" This program is distributed in the hope that it will be useful,
.\" but WITHOUT ANY WARRANTY; without even the implied warranty of
.\" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
.\" GNU General Public License for more details.
.\"
.\" You should have received a copy of the GNU General Public License
.\" along with this program; if not, write to the Free Software
.\" Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
.\" 
.TH sec.pl 1 "October 2001" "Sec 2.0"
.SH NAME
sec.pl \- simple event correlator
.SH SYNOPSIS
.TP 
.B sec.pl
-input=<inputfile>
.br
-log=<logfile>
.br
-pid=<pidfile>
.br
-conf=<conffile pattern> ...
.br
[-input_timeout=<input timeout>]
.br
[-timeout_script=<timeout script>]
.br
[-reopen_timeout=<reopen timeout>]
.br
[-dump=<dumpfile>]
.br
[-cleantime=<clean time>]
.br
[-bufsize=<input buffer size>]
.br
[-quoting | -noquoting]
.br
[-tail | -notail]
.br
[-detach | -nodetach]
.br
[-separator=<separator>]
.SH DESCRIPTION
Sec is a tool that was designed to solve event correlation tasks in 
network and application management. Event correlation is a process where a
stream of primitive events is processed in order to detect
.I composite events
that correspond to event patterns in the event stream.
.PP
After startup sec reads lines from a file, named pipe or standard input,
matches the lines with regular expressions to recognize input events, and 
correlates events according to the rules that are specified in its 
configuration file(s). In order to detect a composite event, rules are used 
sequentially in the same order as they are specified in the configuration 
file. If there are two or more configuration files, rule sequences from 
different files are used in parallel (rules from different files can
interact by creating, using and deleting 
.IR contexts ).
Sec produces its output by executing user-specified shell commands.
By using utilities like
.BR snmptrap (1)
or
.BR snmpnotify (1), 
sec can be configured to generate network management events as output.
.PP
Since sec uses regular expressions for recognizing input events, it can be 
used for matching events from a single logfile, but also for more general
purposes, like correlating a textual event stream on network management server.
.PP
Unlike many other tools, sec does not have a restriction that only one input 
line can be matched by regular expression at a time. 
If desired, multiple consecutive lines can take part in the matching operation
(maximum number of such lines is limited by 
.B -bufsize
parameter).
.PP
Following rule types are currently implemented in sec for detecting 
composite events:
.PP
.B Single
- match input event and immediately execute an action that is 
specified by rule.
.PP
.B SingleWithScript
- match input event and execute an action immediately if external script 
specified by rule returns zero for its exit value.
.PP
.B SingleWithSuppress
- match input event and execute an action 
immediately, but ignore following matching events for the next 
.I t 
seconds.
.PP
.B Pair
- match input event, execute an action immediately, and ignore
following matching events until some other input event arrives. On 
arrival of the second event execute another action.
.PP
.B PairWithWindow
- match input event and wait for
.I t 
seconds for other input event to arrive. If that event is not observed 
within given time window, execute an action. If the event arrives on time, 
execute another action.
.PP
.B SingleWithThreshold
- count matching input events during
.I t
seconds and if given threshold is exceeded, execute an action and ignore
all matching events during rest of the time window.
.PP
.B SingleWith2Thresholds
- count matching input events during 
.I t1
seconds and if given threshold is exceeded, execute an action. Now start 
to count matching events again and if their number per 
.I t2
seconds drops below second threshold, execute another action.
.PP
.B Suppress
- suppress matching input event (used to keep the
event from being matched by later rules)
.PP
.B Calendar
- execute an action at specific times.
.PP
After an event has matched a rule, that could start an event
pattern that defines a composite event; to verify that, this rule
starts a correlation operation that detects this composite event.
If the events following the first event match this pattern, then the
composite event has been found successfully and an action will be executed.
.PP
Rules allow not only shell commands to be executed as actions after
composite events have been detected, but it is also possible to create and 
delete contexts that decide whether a particular rule can be applied at 
the moment, generate new events that will act as input for other rules, and 
reset correlation operations that are performed by other rules. 
This makes it possible to combine several rules and form more complex event 
correlation schemes.
.PP
Contexts can not only be used for "turning off" or "turning on" a particular
rule, but they can also serve as an event stores (in
.BR logsurfer (1)
the contexts are used in a similar way). There are actions for
adding event to a context and reporting all events associated with a
context.
.PP
In order to distinguish between different event correlation operations that
have been started by the rules, sec assings a key to every operation that is 
generated from the rule ID, the name of the configuration file the rule 
belongs to, and textual description of the composite event that the
correlation operation is trying to detect. Since different rules from 
different files might start correlation operations for detecting 
composite events with the same event description strings, the presence of 
rule ID and configuration file name in the keys helps to make a clear 
distinction between these operations.
.SH OPTIONS
.TP 
.B "-input=<inputfile>"
use <inputfile> as event source. <inputfile> can be a regular file, 
named pipe, or standard input if 
.B -
was specified.
.TP 
.B "-log=<logfile>"
use <logfile> for logging sec activities.
.TP 
.B "-pid=<pidfile>"
use <pidfile> for storing process ID of sec.
.TP
.B "-conf=<conffile pattern>"
expand <conffile pattern> to filenames (using perl 
.BR glob () 
function) and read event correlation rules from every file. Multiple
.B -conf
options can be specified on commandline.
.TP 
.B "-input_timeout=<input_timeout>"
.TP 
.B "-timeout_script=<timeout_script>"
if sec has not observed new data in <inputfile> during <input_timeout>
seconds, <timeout_script> will be executed with commandline parameter 1.
If fresh data becomes available again, <timeout_script> will be executed
with commandline parameter 0.
.TP 
.B "-reopen_timeout=<reopen_timeout>"
if sec has not observed new data in <inputfile> during <reopen_timeout> 
seconds, <inputfile> will be reopened.
.TP 
.B "-dump=<dumpfile>"
use <dumpfile> as dumpfile. Default is /tmp/sec.dump (see SIGNALS section 
for more information).
.TP 
.B "-cleantime=<clean_time>"
time interval in seconds that specifies how often internal event and context 
lists are processed to perform time-related tasks and to remove obsolete 
elements. Default is 5 seconds.
.TP 
.B "-bufsize=<input_buffer_size>"
set input buffer to hold last <input_buffer_size> lines that are read from 
<inputfile>. Content of input buffer will be matched by regular expressions 
that are part of rule definitions (i.e., no more than <input_buffer_size>
lines can be matched at a time). Default size is 10 (i.e., 10 lines).
.TP 
.B "-quoting" 
.TP 
.B "-noquoting"
if
.B -quoting
option is specified, all event description strings that
are supplied to external shell commands will be put inside apostrophes. All
apostrophes ' that string originally contains will be replaced with 
similarly looking backticks ` (since backticks will be surrounded by
apostrophes, they will lose their own special meaning). This option prevents 
shell from interpreting special symbols that event description string might 
contain. Default is
.B -noquoting.
.TP 
.B "-tail" 
.TP 
.B "-notail"
if
.B -notail
option is specified, sec will process all data that is 
currently available in <inputfile> and exit when end-of-file is reached. 
Default is
.B -tail
- sec will jump to the end of <inputfile> and wait for new lines to arrive.
With
.B -tail
option, sec follows <inputfile> by its name (and not only by its i-node). If
<inputfile> is recreated or truncated, sec will reopen it and process its
contents from the start. If <inputfile> is removed (i.e., there is
just an i-node left without name), sec will keep the i-node open and wait for
<inputfile> to be recreated. If
.B -reopen_timeout=<reopen_timeout>
option was specified and sec has not observed new data in its input during 
<reopen_timeout> seconds, <inputfile> will be reopened, and if the open 
fails, sec will exit.
.TP 
.B "-detach" 
.TP 
.B "-nodetach"
if
.B -detach
option is specified, sec will disassociate itself from the
controlling terminal. Default is 
.B -nodetach.
.TP 
.B "-separator=<regexp>"
obsolete.
.SH CONFIGURATION FILE
The sec configuration file consists of rule definitions. Each rule
definition is made up of key=value pairs, one key and value per line. 
Values are case sensitive only where character case is important
(like the values specifying regular expressions).
\\-symbol may be used at the end of line to continue the key=value pair
on next line. Lines which begin with #-symbol are treated as comments and 
ignored (whitespace characters may precede #-symbol). Any comment line, 
empty line, or end of the file will terminate the preceding rule definition.
.SS "SINGLE RULE"
The
.B Single
rule was designed for matching input events that require 
immediate action to be taken. Its rule definition has the following
parameters:
.TP 
.I "type"
fixed to Single (value is case insensitive, so single or sIngLe can be
used instead).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive). The first specifies that 
search for matching rules will continue after the match (i.e., matching input 
line(s) will be passed to the next rule), the second will quit the search. 
If 
.I continue
keyword is missing from the rule definition, DontCont is assumed.
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
Specifies the type of the pattern
.I pattern
and how many input
lines the pattern was designed to match. If type begins with RegExp, pattern 
is assumed to be a regular expression. If type begins with SubStr, pattern is
assumed to be a substring that will be searched in input line(s). If
<number> is given, pattern will be compared with last <number> input lines.
If <number> is omitted, 1 is assumed.
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect an event. If regular expression matches input line(s) or
substring is found in input line(s), rule is considered matching and action
is executed. All regular expression constructs that Perl allows are 
allowed here. For example, if case insensitive matching is desired then (?i)
can be prepended to the regular expression (see 
.BR perlre (1)
for more information).
If pattern is a substring then backslash constructs \\t, \\n, \\r, and \\s
can be used to denote tabulation, newline, carriage return and space
character (\\\\ denotes backslash itself).
.TP 
.I "context (optional)"
logical expression that consists of context name(s) combined 
with logical operators !, &&, ||, and parantheses. 
If context <context name> was previously created with 
.I create 
action and has not exceeded its lifetime nor been deleted, the value of 
<context name> is considered to be TRUE.
If the logical expression is true, then the rule matches and action 
is executed; otherwise the rule is considered not matching. 
If
.I pattern 
was a regular expression with bracketing constructs, then 
$1, $2, ... special variables can be used in context name to get the 
backreference values. Special variable $0 can also be used to get the input
line(s) that were matched by
.IR pattern .
.TP 
.I "desc"
textual description of detected event. If 
.I pattern
was a regular expression and bracketing constructs were used inside it, then 
$1, $2, ... special variables can be used in event description to get the 
backreference values. Special variable $0 can also be used to get the input
line(s) that were matched by
.IR pattern .
The event description formed here can be referenced by %s.
.TP 
.I "action"
list of actions separated by semicolons that will be executed when event 
is detected.
Action definition starts by keyword specifying action type, followed by 
additional parameters. If 
.I pattern
was a regular expression and bracketing constructs were used inside it, then 
$1, $2, ... special variables can be used in additional parameters to get the 
backreference values. Special variable $0 can also be used to get the input
line(s) that were matched by
.IR pattern .
Following actions are supported:
.RS
.IP "none"
no action
.IP "logonly"
event description is logged
.IP "shellcmd <shellcmd>"
shell command <shellcmd> is executed. String %s can be used in shell command
to get the value of event description string. If 
.B -quoting
option was specified, %s will be converted to '%s' before supplying it to 
shell command (see 
.B -quoting
and 
.B -noquoting 
options).
.IP "pipe '<event text>' <shellcmd>"
Event <event text> is fed to the standard input of shell command <shellcmd>
(apostrophes are used to mark the beginning and end of the event text, in
order to distinguish it from <shellcmd>). String %s can be used in 
<event text> and <shellcmd> to get the value of event description string.
If <event text> is omitted and there is nothing between apostrophes, %s is 
assumed for <event text> (i.e., event description string is fed to <shellcmd>).
.IP "create [<name> [<time> [<action list>] ] ]
context with the name <name>, with the lifetime of <time> seconds, and with
empty event store is created (<name> may not contain spaces). 
String %s can be used in context name to get the value of event 
description string. If <name> is omitted, %s is assumed for its value 
(i.e., context with the name of event description is created). 
Specifying 0 as <time> or omitting the value means infinite lifetime.
If <action list> is specified, it will be executed when context's lifetime is
exceeded. If <action list> is made up of more than one action, semicolons
must be used to separate the actions, and the list must be enclosed in
parantheses.
If already existing context is recreated with
.IR create , 
its lifetime will be extended for <time> seconds and its event store will 
be emptied.
.IP "delete [<name>]"
context with the name <name> is deleted (<name> may not contain spaces).
String %s can be used in context name to get the value of event 
description string. If <name> is omitted, %s is assumed for its value 
(i.e., context with the name of event description is deleted). 
If non-existing context is deleted, no operation will be performed.
.IP "set <name> <time> [<action list>]
settings for the context <name> will be changed (<name> may not contain 
spaces). New lifetime of the context will be <time> seconds with optional 
<action list> to be executed when the lifetime is exceeded. Event store of 
the context will not be changed by
.IR set .
String %s can be used in context name to get the value of event 
description string. Specifying 0 as <time> means infinite lifetime. 
If <action list> is made up of more than one action, semicolons must be 
used to separate the actions, and the list must be enclosed in parantheses.
.IP "add <name> [<event text>]"
Event <event text> is added to the event store of the context <name> 
(<name> may not contain spaces). Events in the store are ordered by the 
time they were added, and every
.I add
adds event to the end of the store. String %s can be used in <name> and 
<event text> to get the value of event description string. If <event text> 
is omitted, %s is assumed for its value (i.e., event description string is 
added to event store). If context <name> does not exist, the context will
be created with infinite lifetime, empty action list and empty event store
(as with
.I create
<name>) before adding the event.
.IP "report <name> <shellcmd>"
Event store of the context <name> is reported with shell command <shellcmd> 
(<name> may not contain spaces). Reporting means that events from the store
are fed to standard input of <shellcmd> in the order they were added into the
store, every event on a separate line. String %s can be used in 
<name> and <shellcmd> to get the value of event description string.
.IP "event [<time>] [<event text>]"
after <time> seconds new event is fed to input. Feeding means that 
<event text> string is treated exactly like a line read from input - 
it is inserted into the input buffer in order to compare it with rules.
String %s can be used in <event text> to get the value of event 
description string. If <event text> is omitted, %s is assumed for its value 
(i.e., event description string is fed to input). Specifying 0 as <time> or
omitting the value means now.
.IP "reset [<rule_number>] [<event text>]"
cancel all event correlation operations that are currently detecting
the composite event <event text> (if such operations exist). 
If there are multiple configuration files specified, 
.I reset
can cancel only those correlation operations that have been started by the 
rules from the same configuration file where the
.I reset
action itself is defined.
String %s can be used in <event text> to get the value of event description 
string. If <event text> is omitted, %s is assumed for its value (i.e., 
correlation operation that is detecting the composite event equal to event 
description string is cancelled). 
If the correlation operation involves 2 composite events and corresponding
rule definition specifies 2 event descriptions (see 
.I desc
and
.I desc2
parameters of PAIR and PAIRWITHWINDOW rules), then the description of the 
first composite event (that corresponds to
.I desc
parameter) must be used to refer to the correlation operation. 
Since correlation operations started by different rules may detect composite 
events that have identical description strings, rule number can be optionally
specified to point to a correlation operation that was started by specific 
rule (1 means first rule in the configuration file, 2 means second, etc.;
0 denotes the current rule). 
If + or - is prepended to <rule_number>, it is considered to be an offset 
from current rule (for example, -1 means previous rule and +1 next rule).
.RE
.PP
.B Examples:
.PP
type=single
.br
continue=takenext
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]: \\S+ \\(ristov2.*FTP session opened
.br
desc=ftp session opened for ristov2 pid $1
.br
action=create ftp_$1
.PP 
type=single
.br
continue=takenext
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]:
.br
context=ftp_$1
.br
desc=ftp session event for ristov2 pid $1
.br
action=add ftp_$1 $0; set ftp_$1 1800 \\
         (report ftp_$1 /bin/mail root@localhost)
.PP 
type=single
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]: \\S+ \\(ristov2.*FTP session closed
.br
desc=ftp session closed for ristov2 pid $1
.br
action=report ftp_$1 /bin/mail root@localhost; \\
       delete ftp_$1
.PP
First rule creates context with the name
.B ftp_<pid>
when someone connects from host ristov2 with ftp. The second rule adds all 
logfile lines that are associated with the session <pid> to the event store 
of the context
.B ftp_<pid> 
(before adding a line, the rule checks if the context exists). After
adding a line, the rule extends context's lifetime for 30 minutes and sets 
the action list that will be executed when context times out. The third rule 
mails collected logfile lines to root@localhost when the session <pid> is 
closed. Collected lines will also be mailed when the session <pid> has been 
inactive for 30 minutes (no logfile lines observed for that session).
.PP
Note that the logfile line that has matched the first rule will be passed 
to the second rule and will become the first line in the event store 
(the first rule has
.I continue
parameter set to TakeNext). The second rule has also
.I continue
parameter set to TakeNext, since otherwise no logfile lines would reach the 
third rule.
.SS "SINGLEWITHSCRIPT RULE"
The
.B SingleWithScript
rule was designed to integrate external scripts with sec event flow.
Its rule definition is similar to the Single rule, except of additional 
.I script
parameter:
.TP 
.I "type"
fixed to SingleWithScript (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive).
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect an event.
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "script"
script that is executed after 
.I pattern 
has matched some input line(s). If script returns zero for its exit value, 
event has been detected and
.I action
will be executed. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "desc"
textual description of detected event. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that will be executed when 
.I pattern
has matched some input line(s) and 
.I script
returns zero for its exit value. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.PP
.B Examples:
.PP
type=SingleWithScript
.br
ptype=RegExp
.br
pattern=interface (\\S+) down
.br
script=/usr/local/scripts/ping.sh $1
.br
desc=Interface $1 down
.br
action=shellcmd /usr/local/scripts/notify.sh "%s"
.PP
When "interface <ipaddress> down" line appears in input, this rule
acts by calling
.PP
/usr/local/scripts/ping.sh <ipaddress>
.PP
and if the script returns 0 as its exit value, the rule calls
.PP
/usr/local/scripts/notify.sh "Interface <ipaddress> down"
.SS "SINGLEWITHSUPPRESS RULE"
The
.B SingleWithSuppress
rule was designed to implement event correlation
operation called 
.IR compression , 
where multiple instances of the same event are reduced into single event. 
Its rule definition is similar to the Single rule, except of additional 
.I window
parameter:
.TP 
.I "type"
fixed to SingleWithSuppress (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive).
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A.
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "desc"
textual description of event A. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that will be executed when event A is first observed. Following
instances of event A will be ignored. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "window"
time in seconds that following events A are ignored for.
.PP
.B Examples:
.PP
type=SingleWithSuppress
.br
ptype=RegExp
.br
pattern=(\\S+): [fF]ile system full
.br
desc=File system $1 full
.br
action=shellcmd notify.sh "%s"
.br
window=900
.PP
When "/usr: file system full" line appears in input, this rule starts a 
correlation operation that calls
.PP
notify.sh "File system /usr full"
.PP
and ignores following such lines during next 15 minutes. 
.PP
Note that line "/tmp: file system full" would not be ignored, since sec 
identifies correlation operations by a key that contains event
description string of the composite event (given by
.I desc
parameter). Since strings "/usr: file system full" and 
"/tmp: file system full" differ, sec will start another correlation 
operation for the latter.
.SS "PAIR RULE"
The
.B Pair
rule was designed to implement one of the basic forms of 
.I temporal relationship
event correlation operation, where 2 or more
events are reduced into event pair inside given time window. 
Its rule definition has the following parameters:
.TP 
.I "type"
fixed to Pair (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive). Specifies if
input line(s) that match
.I pattern
will be passed to the next rule.
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A.
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "desc"
textual description of event A. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that will be executed when event A is first observed. Following
instances of event A will be ignored. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP
.I "continue2 (optional)"
TakeNext or DontCont (values are case insensitive). Specifies if
input line(s) that match
.I pattern2
will be passed to the next rule.
.TP
.I "ptype2"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern2"
regular expression or substring that input line(s) will be compared with in
order to detect event B. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "context2 (optional)"
context definition. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
.TP 
.I "desc2"
textual description of event B. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
The event description formed here can be referenced by %s.
.TP 
.I "action2"
action list that will be executed when event B is observed. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
.TP 
.I "window (optional)"
time 
.I t
in seconds that is allowed to elapse between first instance
of event A and event B. If event B does not appear during 
.I t
seconds, then the correlation operation started by this rule terminates.
Specifying 0 as value or omitting 
.I window
parameter means setting 
.I t 
to infinity (i.e., if event B does not appear, 
event A will be ignored forever).
.PP
.B Examples:
.PP
type=Pair
.br
ptype=RegExp
.br
pattern=NFS server (\\S+) not responding
.br
desc=$1 is not responding
.br
action=shellcmd notify.sh "%s"
.br
ptype2=substr
.br
pattern2=NFS server $1 ok
.br
desc2=$1 OK
.br
action2=shellcmd notify.sh "%s"
.br
window=3600
.PP
When "NFS server fserv is not responding" line appears in input, the 
correlation operation started by this rule calls
.PP
notify.sh "fserv is not responding"
.PP
and waits for the line "NFS server fserv ok" for 1 hour, ignoring all
"NFS server fserv is not responding" lines. When the line "NFS server fserv
ok" appears, the correlation operation executes shell command
.PP
notify.sh "fserv OK"
.PP
and terminates. If time (1 hour) runs out, the correlation operation will 
terminate without doing anything.
.SS "PAIRWITHWINDOW RULE"
The
.B PairWithWindow
rule was designed to implement another variant of 
.I temporal relationship
event correlation operation that checks if
event A will be followed by event B inside given time window. 
Its rule definition has the following parameters:
.TP 
.I "type"
fixed to PairWithWindow (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive). Specifies if input
line(s) that match
.I pattern
will be passed to the next rule.
.TP
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A.
.TP
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used
in this field.
.TP 
.I "desc"
textual description of event A. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that is executed after event A was observed and event B did not
appear within given time window. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. 
.TP 
.I "continue2 (optional)"
TakeNext or DontCont (values are case insensitive). Specifies if input
line(s) that match
.I pattern2
will be passed to the next rule.
.TP 
.I "ptype2"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern2"
regular expression or substring that input line(s) will be compared with in
order to detect event B. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP
.I "context2 (optional)"
context definition. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
The event description formed here can be referenced by %s.
.TP 
.I "desc2"
textual description of event B. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
The event description formed here can be referenced by %s.
.TP 
.I "action2"
action list that is executed after event A was observed and event B appeared 
within given time window. If 
.I pattern
or 
.I pattern2
parameter was a regular expression, then $0, $1, $2, ... special variables 
can be used in this field. If 
.I pattern2 
was a regular expression, then values of
.I pattern2
are used, otherwise values of 
.I pattern 
are used. 
.br
In the special case where both
.I pattern
and
.I pattern2
were regular expressions, %0, %1, %2, ... special variables can be used to
retrieve values of
.IR pattern ,
while $0, $1, $2, ... refer to values of
.IR pattern2 .
.TP 
.I "window"
size of the time window in seconds.
.PP
.B Examples:
.PP
type=PairWithWindow
.br
ptype=RegExp 
.br
pattern=node (\\S+) interface (\\S+) down
.br
desc=$1 if $2 is down
.br
action=shellcmd notify.sh "%s"
.br
ptype2=SubStr
.br
pattern2=node $1 interface $2 up
.br
desc2=$1 if $2 short outage
.br
action2=event
.br
window=600
.PP
When "node fserv interface 192.168.1.1 down" line appears in input, 
this rule starts a correlation operation that waits 10 minutes for 
"node fserv interface 192.168.1.1 up" line, and if that line does not 
arrive on time, the correlation operation executes shell command
.PP
notify.sh "fserv if 192.168.1.1 is down"
.PP
and terminates. If the line arrives on time, the operation generates event 
"fserv if 192.168.1.1 short outage" and terminates.
.PP
.SS SINGLEWITHTHRESHOLD RULE
The
.B SingleWithThreshold
rule was designed to implement 
.I counting
event correlation operation, where event instances are counted inside
given time window and the number of events is compared with a threshold
value. Its rule definition has the following parameters:
.TP 
.I "type"
fixed to SingleWithThreshold (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive).
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A.
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. 
.TP 
.I "desc"
textual description of event. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that is executed when
.I thresh
instances of event A have been observed within given time window. After that 
all events A will be ignored during rest of the time window. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "window"
size of the time window in seconds. The window is sliding - if event A has 
been observed less than
.I thresh
times at the end of the window, the beginning of the window is moved to the
moment of 2nd appearance of event A, and the counting operation will continue.
If there is no 2nd instance of event A (i.e., event has been observed only
once), the correlation operation will terminate.
.TP 
.I "thresh"
threshold value.
.PP
.B Examples:
.PP
type=SingleWithThreshold
.br
ptype=RegExp
.br
pattern=user (\\S+) login failure on (\\S+)
.br
desc=Repeated login failures for user $1 on $2
.br
action=shellcmd notify.sh "%s"
.br
window=60
.br
thresh=3
.PP
When line "user dbadmin login failure on tty1" is observed, the rule starts
a correlation operation that executes shell command
.PP
notify.sh "Repeated login failures for user dbadmin on tty1"
.PP
if additional two such lines are observed within 1 minute.
Following "user dbadmin login failure on tty1" lines will be ignored until 
60 seconds have elapsed since the arrival of first line. 
.PP
After changing the 
.I action
parameter in the rule definition to
.PP
action=shellcmd notify.sh "%s"; reset 0 %s
.PP
the correlation operation will terminate itself after calling notify.sh, and
next matching line will always start a new counting operation.
.SS "SINGLEWITH2THRESHOLDS RULE"
The
.B SingleWith2Thresholds 
rule was designed to implement another variant 
of 
.I counting
event correlation operation, where event instances are 
counted twice - first for checking if the threshold value is exceeded, and
then for checking if the number of events will stay below the second 
threshold value. Its rule definition has the following parameters:
.TP 
.I "type"
fixed to SingleWith2Thresholds (value is case insensitive).
.TP 
.I "continue (optional)"
TakeNext or DontCont (values are case insensitive).
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A.
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. 
.TP 
.I "desc"
textual description of event. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action"
action list that is executed when
.I thresh
instances of event A have been observed within time window
.IR window .
After that event counting is started again with threshold value 
.I thresh2 
and time window 
.IR window2 . 
If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "window"
size of the first time window in seconds. The window is sliding - if event A 
has been observed less than
.I thresh
times at the end of the window, the beginning of the window is moved to the
moment of 2nd appearance of event A, and the counting operation will continue.
If there is no 2nd instance of event A (i.e., event has been observed only
once), the correlation operation will terminate.
.TP 
.I "thresh"
first threshold value.
.TP 
.I "desc2"
textual description of event. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field. The event description formed here can be referenced by %s.
.TP 
.I "action2"
action list that is executed if no more than 
.I thresh2
events A were observed within last
.I window2 
seconds.
If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.TP 
.I "window2"
size of the second time window in seconds. The window is sliding.
.TP 
.I "thresh2"
second threshold value. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.PP
.B Examples:
.PP
type=SingleWith2Thresholds
.br
ptype=RegExp 
.br
pattern=(\\S+): %SYS-3-CPUHOG
.br
desc=router $1 CPU overload
.br
action=shellcmd notify.sh "%s"
.br
window=300
.br
thresh=2
.br
desc2=router $1 CPU load normal
.br
action2=shellcmd notify.sh "%s"
.br
window2=3600
.br
thresh2=0
.PP
When SYS-3-CPUHOG syslog message is received from a router, the rule starts 
a counting operation that executes shell command
.PP
notify.sh "router <routername> CPU overload"
.PP
if additional SYS-3-CPUHOG syslog message is received from the router within
next 5 minutes. After that the correlation operation waits until no 
SYS-3-CPUHOG syslog messages have been received from the router during last 1 
hour, and then executes
.PP
notify.sh "router <routername> CPU load normal"
.SS "SUPPRESS RULE"
The
.B Suppress
rule was designed to implement 
.I suppression
and
.I filtering
event correlation operations, where event is suppressed or filtered out if
certain context is present or if the event parameters match the pattern 
specified. Its rule definition has the following parameters:
.TP 
.I "type"
fixed to Suppress (value is case insensitive).
.TP 
.I "ptype"
RegExp[<number>] or SubStr[<number>] (values are case insensitive).
.TP 
.I "pattern"
regular expression or substring that input line(s) will be compared with in
order to detect event A (or its subclass) that must be suppressed (or
filtered out).
.TP 
.I "context (optional)"
context definition. If
.I pattern
was a regular expression then $0, $1, $2, ... special variables can be used 
in this field.
.PP
.B Examples:
.PP
type=Suppress
.br
ptype=RegExp
.br
pattern=[Ff]ile system full
.br
context=mycontext
.PP
type=Suppress
.br
ptype=RegExp
.br
pattern=/dev/vg01/\\S+: [Ff]ile system full
.PP
First rule suppresses all "file system full" events if context
.B mycontext
is present. Second rule filters out all "file system full"
events that concern volume group vg01.
.SS "CALENDAR RULE"
The
.B Calendar
rule was designed for executing actions at specific times. Unlike all
other rules, this rule reacts only to the system clock, ignoring other
input. Its rule definition has the following parameters:
.TP 
.I "type"
fixed to Calendar (value is case insensitive).
.TP 
.I "time"
crontab-style time specification (see
.BR crontab (1)
for more information). Time specification consists of five fields separated
by whitespace characters. First field denotes minutes (allowed values 0-59),
second field hours (0-23), third field days (allowed values 0-31, 0 denotes
last day of the month), fourth field months (allowed values 1-12), and fifth
field weekdays (allowed values 0-7, 0 and 7 denotes Sunday). Asterisks (*), 
ranges of numbers (e.g., 8-11), and lists (2,5-6,9) are allowed as field 
values.
.TP 
.I "context (optional)"
context definition.
.TP 
.I "desc"
event description string (can be referenced by %s).
.TP 
.I "action"
action list to be executed.
.PP
.B Examples:
.PP
type=Calendar
.br
time=0 23 * * *
.br
desc=NightContext
.br
action=create %s 32400
.PP
This rule creates the context 
.B NightContext
every day at 11PM. Context has a lifetime of 9 hours.
.SH EXAMPLES
.SS "Example 1"
Consider the following rules:
.PP
type=single
.br
continue=takenext
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]: \\S+ \\(ristov2.*FTP session opened
.br
desc=ftp session opened for ristov2 pid $1
.br
action=create ftp_$1
.PP 
type=single
.br
continue=takenext
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]:
.br
context=ftp_$1
.br
desc=ftp session event for ristov2 pid $1
.br
action=add ftp_$1 $0; set ftp_$1 1800 \\
         (report ftp_$1 /bin/mail root@localhost)
.PP 
type=single
.br
ptype=regexp
.br
pattern=ftpd\\[(\\d+)\\]: \\S+ \\(ristov2.*FTP session closed
.br
desc=ftp session closed for ristov2 pid $1
.br
action=report ftp_$1 /bin/mail root@localhost; \\
       delete ftp_$1
.PP
First rule creates context with the name
.B ftp_<pid>
when someone connects from host ristov2 with ftp. The second rule adds all 
logfile lines that are associated with the session <pid> to the event store 
of the context
.B ftp_<pid> 
(before adding a line, the rule checks if the context exists). After
adding a line, the rule extends context's lifetime for 30 minutes and sets 
the action list that will be executed when context times out. The third rule 
mails collected logfile lines to root@localhost when the session <pid> is 
closed. Collected lines will also be mailed when the session <pid> has been 
inactive for 30 minutes (no logfile lines observed for that session).
.PP
Note that the logfile line that has matched the first rule will be passed 
to the second rule and will become the first line in the event store 
(the first rule has
.I continue
parameter set to TakeNext). The second rule has also
.I continue
parameter set to TakeNext, since otherwise no logfile lines would reach the 
third rule.
.SS "Example 2"
Suppose there is a backup job in your system that runs at 2AM every night
and logs "BACKUP READY" message when it has completed its work. You want to 
send SNMP trap if there is no message in the log by 2:15AM. 
.PP
type=Calendar
.br
time=59 1 * * *
.br
desc=WAITING FOR BACKUP
.br
action=event %s
.PP
type=PairWithWindow
.br
ptype=SubStr
.br
pattern=WAITING FOR BACKUP
.br
desc=Backup not ready!
.br
action=shellcmd sendtrap.sh "%s"
.br
ptype2=SubStr
.br
pattern2=BACKUP READY
.br
desc2=Backup ready
.br
action2=none
.br
window=960
.PP
The first rule generates "WAITING FOR BACKUP" event every night at 1:59AM.
This event is matched by the second rule, which starts a correlation
operation that will wait for "BACKUP READY" event for the next 16 minutes. 
If the event arrives on time, no action is executed, otherwise
.PP
sendtrap.sh "Backup not ready!"
.PP
is called.
.SS "Example 3"
Consider the following rules:
.PP
type=SingleWithScript
.br
ptype=RegExp
.br
pattern=node (\\S+) interface (\\S+) down
.br
script=not_resp.sh $2
.br
desc=NODE $1 IF $2 DOWN
.br
action=event %s
.PP
type=Pair
.br
ptype=RegExp
.br
pattern=NODE (\\S+) IF (\\S+) DOWN
.br
desc=Interface $2 is down at node $1
.br
action=shellcmd notify.sh "%s"
.br
ptype2=SubStr
.br
pattern2=node $1 interface $2 up
.br
desc2=Interface $2 is up at node $1
.br
action2=shellcmd notify.sh "%s"
.br
window=86400
.PP
If "node <node> interface <interface> down" event is observed, the interface
is checked with not_resp.sh script. If the interface is found to be down 
(not_resp.sh returns 0 as its exit code), event 
"NODE <node> IF <interface> DOWN" is generated, which will be matched by the
second rule. The second rule starts a correlation operation that calls
.PP
notify.sh "Interface <interface> is down at node <node>"
.PP
and then waits for the "node <node> interface <interface> up" event
for the next 24 hours. When that event is observed, the correlation
operation calls
.PP
notify.sh "Interface <interface> is up at node <node>"
.PP
and terminates.
.SS "Example 4"
Suppose you have a disk box that logs following error messages:
.PP
.I Description of Error:
.PP
.I Disk drive (CRU: A1) has failed and can no longer be accessed. 
.I (SP Event Code 0xA07)
.PP
.I Probable Cause / Recommended Action:
.PP
.I Replace the disk module (CRU: A1).
.PP
You would like to receive an e-mail message if something happens to the 
disk box. You would like to use different e-mail address at night-time and
also receive a report of all night events. The problem here is that useful 
information is scattered over 7 lines and needs to be consolidated into 
single event. Consider following rules to accomplish this task:
.PP
type=Calendar
.br
time=0 22 * * *
.br
desc=night
.br
action=create %s 36000 \\
       (report %s /bin/mail root@localhost)
.PP
type=Single
.br
ptype=RegExp7
.br
pattern=Description of Error:\\n.*\\n(.+)\\n.*\\n.*\\n.*\\n(.+)
.br
context=night
.br
desc=Error=$1 Recmnd=$2
.br
action=shellcmd nightalarm.sh "%s"; add night %s
.PP
type=Single
.br
ptype=RegExp7
.br
pattern=Description of Error:\\n.*\\n(.+)\\n.*\\n.*\\n.*\\n(.+)
.br
desc=Error=$1 Recmnd=$2
.br
action=shellcmd alarm.sh "%s"
.PP
First rule creates context 
.B night
with the lifetime of 10 hours every day at 10PM.
The second rule specifies that script nightalarm.sh must be used for 
sending alert messages at nights, otherwise script alarm.sh should be used.
Every night-time event is added to context
.BR night ,
and collected events will be mailed to root@localhost at 8AM.
.SS "Example 5"
This section presents an example rulebase for managing Cisco devices.
It is assumed that the managed devices have 
.BR syslog (3)-style 
logging enabled, and that all syslog messages are sent to a central host 
and written to a common logfile that serves as an input for sec.
.PP
# Set up contexts NIGHT and WEEKEND for nights 
.br
# and weekends. Context NIGHT has a lifetime
.br
# of 8 hours and context WEEKEND 2 days
.PP
type=Calendar
.br
time=0 23 * * *
.br
desc=NIGHT
.br
action=create %s 28800
.PP
type=Calendar
.br
time=0 0 * * 6
.br
desc=WEEKEND
.br
action=create %s 172800
.PP
# If a router does not come up within 5 minutes 
.br
# after it was rebooted, generate event 
.br
# "<router> REBOOT FAILURE". The next rule matches 
.br
# this event, checks the router with ping and sends
.br
# a notification if there is no response.
.PP
type=PairWithWindow
.br
ptype=RegExp
.br
pattern=(\\S+) \\d+: %SYS-5-RELOAD
.br
desc=$1 REBOOT FAILURE
.br
action=event %s
.br
ptype2=RegExp
.br
pattern2=($1) \\d+: %SYS-5-RESTART
.br
desc2=$1 successful reboot
.br
action2=logonly
.br
window=300
.PP
type=SingleWithScript
.br
ptype=RegExp
.br
pattern=(\\S+) REBOOT FAILURE
.br
script=not_responding.sh $1
.br
desc=$1 did not come up after reboot
.br
action=shellcmd notify.sh "%s"
.PP
# Send a notification if CPU load of a router is too 
.br
# high (two CPUHOG messages are received within 5 
.br
# minutes); send another notification if the load is 
.br 
# normal again (no CPUHOG messages within last 15 
.br
# minutes). Rule is not applied at night or weekend.
.PP
type=SingleWith2Thresholds
.br
ptype=RegExp
.br
pattern=(\\S+) \\d+: %SYS-3-CPUHOG
.br
context=!(NIGHT || WEEKEND)
.br
desc=$1 CPU overload
.br
action=shellcmd notify.sh "%s"
.br
window=300
.br
thresh=2
.br
desc2=$1 CPU load normal
.br
action2=shellcmd notify.sh "%s"
.br
window2=900
.br
thresh2=0
.PP
# If a router interface is in down state for less 
.br
# than 15 seconds, generate event 
.br
# "<router> INTERFACE <interface> SHORT OUTAGE"; 
.br
# otherwise generate event 
.br
# "<router> INTERFACE <interface> DOWN".
.PP
type=PairWithWindow
.br
ptype=RegExp
.br
pattern=(\\S+) \\d+: %LINK-3-UPDOWN: Interface (.+), changed state to down
.br
desc=$1 INTERFACE $2 DOWN
.br
action=event %s
.br
ptype2=RegExp
.br
pattern2=($1) \\d+: %LINK-3-UPDOWN: Interface ($2), changed state to up
.br
desc2=$1 INTERFACE $2 SHORT OUTAGE
.br
action2=event %s
.br
window=15
.PP
# If "<router> INTERFACE <interface> DOWN" event is 
.br
# received from previous rule, send a notification and 
.br
# wait for "interface up" event for the next 24 hours
.PP
type=Pair
.br
ptype=RegExp
.br
pattern=(\\S+) INTERFACE (\\S+) DOWN
.br
desc=$1 interface $2 is down
.br
action=shellcmd notify.sh "%s"
.br
ptype2=RegExp
.br
pattern2=($1) \\d+: %LINK-3-UPDOWN: Interface ($2), changed state to up
.br
desc2=$1 interface $2 is up
.br
action2=shellcmd notify.sh "%s"
.br
window=86400
.PP
# If ten "short outage" events have been observed 
.br
# in the window of 6 hours, send a notification
.PP
type=SingleWithThreshold
.br
ptype=RegExp
.br
pattern=(\\S+) INTERFACE (\\S+) SHORT OUTAGE
.br
desc=Interface $2 at node $1 is unstable
.br
action=shellcmd notify.sh "%s"
.br
window=21600
.br
thresh=10
.SH SIGNALS
.TP 
.B "SIGHUP"
sec will reload its configuration, reopen its input and logfile, and reset 
internal lists that contain correlation information (i.e., all active event
correlation operations will be cancelled and all contexts will be deleted).
.TP
.B "SIGABRT"
sec will act like it has received
.BR SIGHUP ,
but contexts (and their event stores) will not be deleted. Note that on some
systems
.B SIGIOT
is used in place of
.BR SIGABRT .
.TP 
.B "SIGUSR1"
some information about current state of sec (content of internal lists, rule
usage statistics, etc.) will be written into dumpfile.
.TP 
.B "SIGUSR2"
sec will reopen its logfile. Useful for logfile rotating, since it 
does not cancel active event correlation operations like
.B SIGHUP
does.
.SH AUTHOR
Risto Vaarandi <risto.vaarandi@eyp.ee>
.SH "SEE ALSO"
.BR crontab (1),
.BR logsurfer (1),
.BR perl (1),
.BR perlre (1),
.BR snmpnotify (1),
.BR snmptrap (1),
.BR syslog (3)
